#!/usr/bin/php
<?php

include_once "include/vars.php";
include_once "include/config.php";
include_once "include/import.php";
include_once "$theme/theme.php";

include_once "include/common.php";
include_once "include/site.php";
include_once "include/exif.php";
include_once "include/color.php";
include_once "include/database.php";

// Tell the world who we are.
printf ("Photo Organiser Command Line Bulk Upload Tool v.0.0.1\n\n");

// First let the user log on:
//
// We do not need a password, but we want to upload the files under a specific
// account other than administrator.
//
printf ("Please enter user name for bulk upload: ");
fscanf(STDIN, "%s", $username);

// Open database:
$database = po_dbconnect();

//
// Ask for user name as whom we need to do the bulk upload.
//

$user_data = pg_fetch_row(pg_query($database, "select identifier, first_name, last_name from users where username='$username'"));

if ($user_data[0] != 0) {
  $user_id=$user_data[0];
  $first_name=$user_data[1];
  $last_name=$user_data[2];
  printf ("Hello, %s %s [%d].\n", $first_name, $last_name, $user_id);
} else {
  printf ("Sorry, user '%s' does not exist.\n", $username);
  die;
}


//
// display user's folder tree.
//

find_folder_tree($database, $user_id, "", $folder_list, 0);

printf ("\nHierarchical folder list:\n");
for ($i = 1; $i <= count($folder_list); $i++) {
  $indent = str_repeat("  ", $folder_list[$i]["level"]);
  printf("[%4d] %s%s <id=%d>\n", $i, $indent, $folder_list[$i]["caption"], $folder_list[$i]["id"]);
}

printf ("Total of %d folders.\n", count($folder_list));


//
// Ask user for folder in which to create the new folders.
//

do {
  printf ("\nEnter number of folder in which to upload the local structure: ");
  fscanf(STDIN, "%s", $folder_key);

  if (($folder_key < 0) or ($folder_key > count($folder_list))) {
     printf ("Sorry, that folder number does not exist. Please try again.\n");
  }
} while (($folder_key < 0) or ($folder_key > count($folder_list)));


if ( $folder_key == 0 ) {
  printf ("You selected the root folder.\n");
  $root_folder_id = "null";
  $root_folder_name = "/";
} else {
  // printf ("You selected the folder '%s'.\n", $folder_list[$folder_key]["caption"]);
  printf ("You selected the folder '%s/%s'.\n", get_path_to_folder_slashed($database, $folder_list[$folder_key]["id"]), $folder_list[$folder_key]["caption"]);
  $root_folder_id = $folder_list[$folder_key]["id"];
  $root_folder_name = get_path_to_folder_slashed($database, $folder_list[$folder_key]["id"]) . "/" . $folder_list[$folder_key]["caption"];
}


//
// Display directory tree.
//

$path = getcwd();
find_directory_tree (".", $directory_list, 0);

printf ("\nHierarchical directory list under '%s':\n", $path);
for ($i = 1; $i <= count($directory_list); $i++) {
  $indent = str_repeat("  ", $directory_list[$i]["level"]);
  printf("[%4d] %s%s <id=%d, p_key=%d, p_id=%d>\n", $i, $indent, $directory_list[$i]["name"], $directory_list[$i]["id"], $directory_list[$i]["parent_key"], $directory_list[$i]["parent_id"]);
}


//
// Ask confirmation before adding new folders.
//

do {
  printf ("\nShall I create this folder structure in '%s'? (y/n): ", $root_folder_name);
  fscanf(STDIN, "%s", $answer);

  if (($answer != "y") and ($answer != "n")) {
     printf ("Please answer 'y' or 'n'.\n");
  }
} while (($answer != "y") and ($answer != "n"));


//
// Create folders if answer was positive.
//

if ($answer == "y") {
  printf ("Creating folder structure in '%s'...\n", $root_folder_name);

  for ($i = 1; $i <= count($directory_list); $i++) {

    // print what we are doing
    $indent = str_repeat("  ", $directory_list[$i]["level"]);
    printf("Creating folder %s%s\n", $indent, $directory_list[$i]["name"]);

    // begin transaction
    pg_query($database, "begin");

    // Get a new unique ID for this folder.
    $result = pg_fetch_row(pg_query($database, "select nextval('folder_id_sequence')"));
    $directory_list[$i]["id"]=$result[0];

    // Retrieve ID from parent
    $directory_list[$i]["parent_id"] = $directory_list[$directory_list[$i]["parent_key"]]["id"];

    // Find out parent folder
    if ($directory_list[$i]["parent_id"] == 0 ) {
      $parent_id = $root_folder_id;
    } else {
      $parent_id = $directory_list[$i]["parent_id"];
    }

    // shortcut names for values
    $folder_id = $directory_list[$i]["id"];
    $caption = $directory_list[$i]["name"];

    // insert folder
    $result = pg_query($database, "insert into folder (identifier, parent_folder, users, caption, description, date_of_creation, access_rights, orderby) values ($folder_id, $parent_id, $user_id, '$caption', 'CLI generated folder', now(), 3, 1)");

    // commit transaction
    pg_query($database, "commit");

  }

} else {
  printf ("Skipping folder structure create.\n");
}






function find_directory_tree ($path, &$directory_list, $level )
{
  // if no path specified, assume current directory
  if ( $path == "" ) {
    $path = ".";
  }

  if (!file_exists($path)) {
    echo "not an existing directory";
    return;
  }

  if (!is_dir($path)) {
    echo $path." isnt a directory";
    return;
  }

  $parent_key = count($directory_list);

  $main_dir =opendir ($path) or die("Error reading ".$path);


  while ($entry=readdir($main_dir))
  {
    // Only look for directories
    if (filetype($path."/".$entry) == "dir")
    {
      // Ignore special directories "." and ".."
      if ($entry != "." and  $entry != "..")
      {
        $count = count($directory_list)+1;
        $directory_list[$count]["name"]=$entry;
        $directory_list[$count]["path"]=$path."/".$entry;
        $directory_list[$count]["level"]=$level;
        $directory_list[$count]["parent_key"]=$parent_key;
        $directory_list[$count]["id"]=0;
        $directory_list[$count]["parent_id"]=0;

        find_directory_tree($path."/".$entry, $directory_list, $level+1);
      }
    }
  }
}




function find_folder_tree($database, $user_id, $parent_id, &$folder_list, $level )
{
  // If user_id is empty, assume administrator
  if ($user_id == "") {
    $user_id = 1;
  }

  // Select only subfolders of the specified folder, if no folder is specified
  // we assume the root folder is requested
  if ($parent_id == "") {
    $folder_selector = "parent_folder is null";
  }
  else {
    $folder_selector = "parent_folder=$parent_id";
  }

  $parent_key = count($folder_list);

  // Get the list of subfolders.
  $sub_folders = pg_query($database, "select identifier, caption from folder where users=$user_id and $folder_selector order by date_of_creation asc");

  // Get the number of subfolders
  $num_of_folders = pg_num_rows($sub_folders);

  // Loop over all subfolders, and recurse into them if they have subfolders
  // themselves.
  for ($i = 0; $i < $num_of_folders; $i++) {
    $folder = pg_fetch_row($sub_folders, $i);

    // We filter out the Trash and Orphanage folders.
    if (($folder[1] != "Trash" && ($folder[1] != "Orphanage")) {
      $count = count($folder_list)+1;
      $folder_list[$count]["id"]=$folder[0];
      $folder_list[$count]["caption"]=$folder[1];
      $folder_list[$count]["level"]=$level;
      $folder_list[$count]["parent_id"]=$parent_id;
      $$folder_list[$count]["parent_key"]=$parent_key;
      $subfolders_in_folder = pg_fetch_row(pg_query($database, "select count_subfolders_by_folder($folder[0])"));
      if ($subfolders_in_folder > 0 ) {
        find_folder_tree($database, $user_id, $folder[0], $folder_list, $level+1);
      }
    }
  }
}




function get_path_to_folder_slashed($database, $folder_id) {
  $stack_size = 0;

  $folder_data = pg_fetch_row(pg_query($database, "
      select parent_folder, folder.caption
        from folder
       where folder.identifier = '$folder_id'"));
  $folder_id = $folder_data[0];

  if ($folder_id == "")
    return;

  do {
    $folder_data = pg_fetch_row(pg_query($database, "
      select parent_folder, folder.caption
        from folder
       where folder.identifier = '$folder_id'"));
    $folder_data[1] = strip_tags($folder_data[1]);
    $folder_data[1] = makeZanza($folder_data[1], "No caption");
    $stack[$stack_size] = "$folder_data[1]";
    $folder_id = $folder_data[0];
    $stack_size++;
} while ($folder_id);

  $path = "";
  for ($i = $stack_size - 1; $i >= 0; $i--) {
    $path = $path . "/$stack[$i]";
}

  return $path;
}

?>
