<?php

//  Copyright (C) 2002, 2003, 2004, 2005, 2006 Balint Kis (balint@k-i-s.net)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

$iptc_field_names = array (5 => "Object Name",
                           7 => "Edit Status",
                           10 => "Priority",
                           15 => "Category",
                           20 => "Supplemental Category",
                           22 => "Job Identifier",
                           25 => "Keywords",
                           30 => "Release Date",
                           35 => "Release Time",
                           40 => "Special Instructions",
                           55 => "Created Date",
                           60 => "Created Time",
                           65 => "Originating Program",
                           70 => "Program Version",
                           80 => "Byline",
                           85 => "Byline Title",
                           90 => "City",
                           95 => "Province, State",
                           100 => "Country Code",
                           101 => "Country",
                           103 => "Org. Transmission Reference",
                           105 => "Headline",
                           110 => "Credit",
                           120 => "Caption",
                           121 => "Local Caption",
                           122 => "Caption Writer");

function human_readable_iptc($iptc_string) {
  global $iptc_field_names;

  /* this parser replaces unhandled iptc fields */
  /* we leave the iptc numbers in the database so that */
  /* we have a chance to add them in the future */

  $iptc_string = ereg_replace("IPTC ", "", $iptc_string);

  $key_start_position = 0;
  $new_iptc_string = "";

  do {
    $key_start_position = strpos($iptc_string, "<tr><td>", $key_start_position) + 8;
    $key_end_position = strpos($iptc_string, "</td><td>", $key_start_position);

    $value_start_position = $key_end_position + 9;
    $value_end_position = strpos($iptc_string, "</td></tr>", $value_start_position);

    $key = substr($iptc_string, $key_start_position, $key_end_position - $key_start_position);
    $key = trim(substr($key, 0, strpos($key, ".")));
    $key += 0.0;
    $value = substr($iptc_string, $value_start_position, $value_end_position - $value_start_position);

    $tail = substr($iptc_string, $value_end_position + 10);

    if ($iptc_field_names[$key])
      $new_iptc_string = $new_iptc_string ."<tr><td>". $iptc_field_names[$key] ."</td><td>".$value ."</td></tr>";
  } while ($tail != "");

  return $new_iptc_string;
}

function convert_iptc2sql_timestamp($date, $time) {
  $year   = substr($date,  0, 4);
  $month  = substr($date,  4, 2);
  $day    = substr($date,  6, 2);

  if ($date == "" || $year == 0 || $month == 0 || $day == 0) {
    $today = getdate();
    $year   = $today['year'];
    $month  = $today['mon'];
    $day    = $today['mday'];
  }

  $hour   = substr($time,  0, 2);
  $minute = substr($time,  2, 2);
  $second = substr($time,  4, 2);

  if ($date == "") {
    $hour   = "00";
    $minute = "00";
    $second = "00";
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;
  return $sql_timestamp;
}

function convert_iptc_location($database, $user_id, $city, $state, $country) {
  $location = pg_fetch_row(pg_query($database, "
     select location.identifier
       from location, country
      where '$city' ~*  location.city
        and '$state' ~* state.value
        and location.state = state.identifier
        and '$country' ~*  country.value
        and location.country = country.identifier"));

  /* unlike elsewhere we look for exact matches */
  if ($location) {
    return $location[0];
  }

  /* if no location found create a new one in the database */
  pg_query($database, "begin");

  /* check for existing state/region */
  /* if not found create one */
  if (($state) && ($state != 'null')) {
    $state_id = pg_fetch_row(pg_query($database, "select identifier from state where value ~* '$state'"));
    if (!$state_id) {
      $state_id = pg_fetch_row(pg_query($database, "select nextval('state_id_sequence')"));
      if (!$state_id) {
        pg_query($database, "rollback");
        return "null";
      }
      pg_query($database, "insert into state (identifier, value) values ($state_id[0], '$state')");
    }
  }

  /* check for existing country */
  /* if not found create one - you never know these days */
  if (($country) && ($country != 'null')) {
    $country_id = pg_fetch_row(pg_query($database, "select identifier from country where value ~* '$country'"));
    if (!$country_id) {
      $country_id = pg_fetch_row(pg_query($database, "select nextval('country_id_sequence')"));
      if (!$country_id) {
        pg_query($database, "rollback");
        return "null";
      }
      pg_query($database, "insert into country (identifier, value) values ($country_id[0], '$country')");
      print "insert into country (identifier, value) values ($country_id[0], '$country'";
    }
  }

  $new_location_id = pg_fetch_row(pg_query($database, "select nextval('location_id_sequence')"));
  if (!$new_location_id) {
    pg_query($database, "rollback");
    return "null";
  }

  $state_id[0] = $state_id[0] ? $state_id[0] : "null";
  $country_id[0] = $country_id[0] ? $country_id[0] : "null";

  $result = pg_query($database, "insert into location
                                        (identifier, city, state, country, last_modified_date, last_modifying_users)
                                 values ($new_location_id[0], '$city', $state_id[0], $country_id[0], now(), $user_id)");
  if (!$result) {
    pg_query($database, "rollback");
    return "null";
  }

  pg_query($database, "commit");

  return $new_location_id[0];
}

function weird_to_extended_ascii($s) {
  $conversion_table = array(
                            128 => "",
                            129 => "",
                            130 => "",
                            131 => "",
                            132 => "",
                            133 => "",
                            134 => "",
                            135 => "",
                            136 => "",
                            137 => "",
                            138 => "",
                            139 => "",
                            140 => "",
                            141 => "",
                            142 => "",
                            143 => "",
                            144 => "",
                            145 => "",
                            146 => "",
                            147 => "",
                            148 => "",
                            149 => "",
                            150 => "",
                            151 => "",
                            152 => "",
                            153 => "",
                            154 => "",
                            155 => "",
                            156 => "",
                            157 => "",
                            158 => "",
                            159 => "",
                            163 => "",
                            164 => "",
                            166 => "",
                            167 => "",
                            168 => "",
                            169 => "",
                            174 => "",
                            175 => "",
                            180 => "",
                            181 => "",
                            190 => "",
                            191 => "",
                            192 => "",
                            203 => "",
                            204 => "",
                            205 => "",
                            206 => "",
                            207 => "",
                            229 => "",
                            230 => "",
                            231 => "",
                            232 => "",
                            233 => "",
                            234 => "",
                            235 => "",
                            236 => "",
                            237 => "",
                            238 => "",
                            239 => "",
                            241 => "",
                            242 => "",
                            243 => "",
                            244 => ""
                            );

  $len = strlen($s);
  for($x = 0; $x < $len; $x++) {
    if (ord($s[$x]) < 128) {
      $result .= $s[$x];
    } else {
      if ($conversion_table[ord($s[$x])]) {
        $result .= $conversion_table[ord($s[$x])];
      }
      else {
        $result .= ord($s[$x]);
      }
    }
  }
  return $result;
}

function validate_iptc_string($iptc_string) {
  $string_length = strlen($iptc_string);
  $newstring = "";

  /* Remove non displayable ASCII characters */
  for ($i=0; $i < $string_length; $i++) {
    $ascii_num = ord($iptc_string[$i]);

    if ($ascii_num == 0x27)
      continue;

    if ((0xff > $ascii_num) && ($ascii_num > 0x1F))
      $newstring = $newstring . $iptc_string[$i];
  }

  $newstring = weird_to_extended_ascii($newstring);

  /* Check for null fields. All strings containing spaces only are */
  /* condsidered null, just like dates of format 0000:00:00 00:00:00 */
  /* therefore if a string contains only zero, column or space */
  /* return an empty string */

  $trans = array(" " => "", ":" => "", "-" => "", "0" => "");
  $stripped_string = strtr($newstring, $trans);
  if ($stripped_string == "")
    return $stripped_string;

  return $newstring;
}

function photo_parse_iptc($database, $index, $user_id, $image_data) {
  error_reporting(0);
  $size = getimagesize($image_data['file'][$index]['name_tmp'], $info);
  error_reporting(1);

  $city = "";
  $state = "";
  $country = "";

  $creation_date = "";
  $creation_time = "";

  if (!isset($info["APP13"]))
    return $image_data;

  $iptc = iptcparse ($info["APP13"]);
  if ($iptc === FALSE)
    return $image_data;

  print "<li>Extracting IPTC data ... ";

  $image_data['keyword'] = "";
  $image_data['title'] = "";
  $image_data['author'] = "";
  $image_data['caption'] = "";
  $image_data['copyright'] = "";
  $image_data['iptc'] = "";

  foreach ($iptc as $key=>$section) {
    $key = substr($key, strrpos($key, "#") + 1);
    foreach ($section as $name=>$val) {
      $val = validate_iptc_string($val);

      if ($val != "") {
        switch ($key) {
        case 5:
          if ($image_data['title'] == "")
            $image_data['title'] = $val;
          else
            $image_data['title'] .= ", $val";
          break;
        case 15:
          $image_data['category'] = substr($val, 0, 3);
          break;
        case 20:
          if ($image_data['supplemental_category'] == "")
            $image_data['supplemental_category'] = $val;
          else
            $image_data['supplemental_category'] .= ", $val";
          break;
        case 25:
          if ($image_data['keyword'] == "")
            $image_data['keyword'] = $val;
          else
            $image_data['keyword'] .= "; $val";
          break;
        case 40:
          if ($image_data['instructions'] == "")
            $image_data['instructions'] = $val;
          else
            $image_data['instructions'] .= ", $val";
          break;
        case 80:
          if ($image_data['author'] == "")
            $image_data['author'] = $val;
          else
            $image_data['author'] .= ", $val";
          break;
        case 55:
          $creation_date = $val;
          break;
        case 60:
          $creation_time = $val;
          break;
        case 90:
          $city = $val;
          break;
        case 95:
          $state = $val;
          break;
        case 101:
          $country = $val;
          break;
        case 105:
          if ($image_data['headline'] == "")
            $image_data['headline'] = $val;
          else
            $image_data['headline'] .= ", $val";
          break;
        case 103:
          if ($image_data['transmission_reference'] == "")
            $image_data['transmission_reference'] = $val;
          else
            $image_data['transmission_reference'] .= ", $val";
          break;
        case 110:
          if ($image_data['credit'] == "")
            $image_data['credit'] = $val;
          else
            $image_data['credit'] .= ", $val";
          break;
        case 115:
          if ($image_data['source'] == "")
            $image_data['source'] = $val;
          else
            $image_data['source'] .= ", $val";
          break;
        case 116:
          if ($image_data['copyright'] == "")
            $image_data['copyright'] = $val;
          else
            $image_data['copyright'] .= ", $val";
          break;
        case 120:
          if ($image_data['caption'] == "")
            $image_data['caption'] = $val;
          else
            $image_data['caption'] .= ", $val";
          break;
        case 122:
          if ($image_data['caption_writer'] == "")
            $image_data['caption_writer'] = $val;
          else
            $image_data['caption_writer'] .= ", $val";
          break;
        default:
          $image_data['iptc'] .= "<tr><td>$key.$name</td><td>$val</td></tr>";
        }
      }
    }
  }

  if (($city != "") || ($state != "") || ($country != ""))
    $image_data['location'] = convert_iptc_location($database, $user_id, $city, $state, $country);

  if (($creation_date != "") || ($creation_time != ""))
    $image_data['date_of_exposure'] = convert_iptc2sql_timestamp($creation_date, $creation_time);

  print "done</li>";

  return $image_data;
}

?>
